@@ -17,45 +17,45 @@
 
 from typing import Optional
 
-from nbxmpp.namespaces import Namespace
-from nbxmpp.protocol import ERR_NOT_ACCEPTABLE
-from nbxmpp.protocol import JID
-from nbxmpp.protocol import Message
-from nbxmpp.protocol import DataForm
-from nbxmpp.protocol import DataField
-from nbxmpp.protocol import NodeProcessed
-from nbxmpp.protocol import StanzaMalformed
-from nbxmpp.structs import MucSubject
-from nbxmpp.structs import StanzaHandler
 from nbxmpp.const import InviteType
 from nbxmpp.const import MessageType
 from nbxmpp.const import StatusCode
-from nbxmpp.structs import DeclineData
-from nbxmpp.structs import InviteData
-from nbxmpp.structs import VoiceRequest
-from nbxmpp.structs import AffiliationResult
-from nbxmpp.structs import MucConfigResult
-from nbxmpp.structs import MucDestroyed
-from nbxmpp.task import iq_request_task
 from nbxmpp.errors import is_error
 from nbxmpp.errors import StanzaError
-from nbxmpp.modules.util import raise_if_error
-from nbxmpp.modules.util import parse_xmpp_uri
-from nbxmpp.modules.util import process_response
-from nbxmpp.modules.dataforms import extend_form
 from nbxmpp.modules.base import BaseModule
-from nbxmpp.modules.muc.util import MucInfoResult
-from nbxmpp.modules.muc.util import make_affiliation_request
-from nbxmpp.modules.muc.util import make_destroy_request
-from nbxmpp.modules.muc.util import make_set_config_request
-from nbxmpp.modules.muc.util import make_config_request
-from nbxmpp.modules.muc.util import make_cancel_config_request
-from nbxmpp.modules.muc.util import make_set_affiliation_request
-from nbxmpp.modules.muc.util import make_set_role_request
-from nbxmpp.modules.muc.util import make_captcha_request
+from nbxmpp.modules.dataforms import extend_form
 from nbxmpp.modules.muc.util import build_direct_invite
 from nbxmpp.modules.muc.util import build_mediated_invite
+from nbxmpp.modules.muc.util import make_affiliation_request
+from nbxmpp.modules.muc.util import make_cancel_config_request
+from nbxmpp.modules.muc.util import make_captcha_request
+from nbxmpp.modules.muc.util import make_config_request
+from nbxmpp.modules.muc.util import make_destroy_request
+from nbxmpp.modules.muc.util import make_set_affiliation_request
+from nbxmpp.modules.muc.util import make_set_config_request
+from nbxmpp.modules.muc.util import make_set_role_request
+from nbxmpp.modules.muc.util import MucInfoResult
 from nbxmpp.modules.muc.util import parse_muc_user
+from nbxmpp.modules.util import parse_xmpp_uri
+from nbxmpp.modules.util import process_response
+from nbxmpp.modules.util import raise_if_error
+from nbxmpp.namespaces import Namespace
+from nbxmpp.protocol import DataField
+from nbxmpp.protocol import DataForm
+from nbxmpp.protocol import ERR_NOT_ACCEPTABLE
+from nbxmpp.protocol import JID
+from nbxmpp.protocol import Message
+from nbxmpp.protocol import NodeProcessed
+from nbxmpp.protocol import StanzaMalformed
+from nbxmpp.structs import AffiliationResult
+from nbxmpp.structs import DeclineData
+from nbxmpp.structs import InviteData
+from nbxmpp.structs import MucConfigResult
+from nbxmpp.structs import MucDestroyed
+from nbxmpp.structs import MucSubject
+from nbxmpp.structs import StanzaHandler
+from nbxmpp.structs import VoiceRequest
+from nbxmpp.task import iq_request_task
 
 
 class MUC(BaseModule):
@@ -63,7 +63,7 @@ class MUC(BaseModule):
     _depends = {
         'disco_info': 'Discovery',
         'request_vcard': 'VCardTemp',
-        'send_retract_request': 'Moderation',
+        'send_moderation_request': 'Moderation',
     }
 
     def __init__(self, client):
@@ -79,6 +79,10 @@ class MUC(BaseModule):
                           callback=self._process_muc_user_presence,
                           ns=Namespace.MUC_USER,
                           priority=11),
+            StanzaHandler(name='message',
+                          callback=self._process_message_base,
+                          ns=Namespace.MUC_USER,
+                          priority=5),
             StanzaHandler(name='message',
                           callback=self._process_message_before_decryption,
                           typ='groupchat',
@@ -211,16 +215,6 @@ class MUC(BaseModule):
 
         properties.occupant_id = occupant_id
 
-        muc_user = stanza.getTag('x', namespace=Namespace.MUC_USER)
-        if muc_user is not None:
-            try:
-                properties.muc_user = parse_muc_user(muc_user,
-                                                     is_presence=False)
-            except StanzaMalformed as error:
-                self._log.warning(error)
-                self._log.warning(stanza)
-                raise NodeProcessed
-
         addresses = stanza.getTag('addresses', namespace=Namespace.ADDRESS)
         if addresses is not None:
             address = addresses.getTag('address', attrs={'type': 'ofrom'})
@@ -240,6 +234,19 @@ class MUC(BaseModule):
             author=properties.muc_nickname,
             timestamp=properties.user_timestamp)
 
+    def _process_message_base(self, _client, stanza, properties):
+        muc_user = stanza.getTag('x', namespace=Namespace.MUC_USER)
+        if muc_user is None:
+            return
+
+        try:
+            properties.muc_user = parse_muc_user(muc_user,
+                                                 is_presence=False)
+        except StanzaMalformed as error:
+            self._log.warning(error)
+            self._log.warning(stanza)
+            raise NodeProcessed
+
     def _process_message(self, _client, stanza, properties):
         muc_user = stanza.getTag('x', namespace=Namespace.MUC_USER)
         if muc_user is None:
@@ -506,9 +513,14 @@ class MUC(BaseModule):
         response = yield make_set_role_request(room_jid, nick, role, reason)
         yield process_response(response)
 
-    def retract_message(self, room_jid: JID, stanza_id: str,
-                        reason: Optional[str] = None) -> None:
-        self.send_retract_request(room_jid, stanza_id, reason)
+    def moderate_message(
+        self,
+        namespace: str,
+        room_jid: JID,
+        stanza_id: str,
+        reason: Optional[str] = None
+    ) -> None:
+        self.send_moderation_request(namespace, room_jid, stanza_id, reason)
 
     def set_subject(self, room_jid, subject):
         message = Message(room_jid, typ='groupchat', subject=subject)
